/* Copyright Statement:
 *
 * (C) 2005-2016  MediaTek Inc. All rights reserved.
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein
 * is confidential and proprietary to MediaTek Inc. ("MediaTek") and/or its licensors.
 * Without the prior written permission of MediaTek and/or its licensors,
 * any reproduction, modification, use or disclosure of MediaTek Software,
 * and information contained herein, in whole or in part, shall be strictly prohibited.
 * You may only use, reproduce, modify, or distribute (as applicable) MediaTek Software
 * if you have agreed to and been bound by the applicable license agreement with
 * MediaTek ("License Agreement") and been granted explicit permission to do so within
 * the License Agreement ("Permitted User").  If you are not a Permitted User,
 * please cease any access or use of MediaTek Software immediately.
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT MEDIATEK SOFTWARE RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES
 * ARE PROVIDED TO RECEIVER ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NONINFRINGEMENT.
 * NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH RESPECT TO THE
 * SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY, INCORPORATED IN, OR
 * SUPPLIED WITH MEDIATEK SOFTWARE, AND RECEIVER AGREES TO LOOK ONLY TO SUCH
 * THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO. RECEIVER EXPRESSLY ACKNOWLEDGES
 * THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES
 * CONTAINED IN MEDIATEK SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK
 * SOFTWARE RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S ENTIRE AND
 * CUMULATIVE LIABILITY WITH RESPECT TO MEDIATEK SOFTWARE RELEASED HEREUNDER WILL BE,
 * AT MEDIATEK'S OPTION, TO REVISE OR REPLACE MEDIATEK SOFTWARE AT ISSUE,
 * OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE CHARGE PAID BY RECEIVER TO
 * MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 */

#ifndef __HAL_UART_INTERNAL_H__
#define __HAL_UART_INTERNAL_H__

#ifdef HAL_UART_MODULE_ENABLED

#include <assert.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
    UART_INTERRUPT_NONE,
    UART_INTERRUPT_RECEIVE_ERROR,
    UART_INTERRUPT_RECEIVE_TIMEOUT,
    UART_INTERRUPT_RECEIVE_BREAK,
    UART_INTERRUPT_RECEIVE_AVAILABLE,
    UART_INTERRUPT_SEND_AVAILABLE,
    UART_INTERRUPT_SOFTWARE_FLOWCONTROL,
    UART_INTERRUPT_HARDWARE_FLOWCONTROL
} uart_interrupt_type_t;

typedef struct {
    uint32_t RBR; /* RX buffer register */
    uint32_t THR; /* TX holding register */
    uint32_t DLL; /* Divisor latch register */
    uint32_t IER; /* Interrupt enable register */
    uint32_t DLM; /* Divisor latch register */
    uint32_t IIR; /* Interrupt identification register */
    uint32_t FCR; /* FIFO control register */
    uint32_t EFR; /* Enhanced feature register */
    uint32_t LCR; /* Line control register */
    uint32_t MCR; /* Modem control register */
    uint32_t XON1; /* XON1 char register */
    uint32_t LSR; /* Line status register */
    uint32_t XON2; /* XON2 char register */
    uint32_t XOFF1; /* XOFF1 char register */
    uint32_t SCR; /* Scratch register */
    uint32_t XOFF2; /* XOFF2 char register */
    uint32_t AUTOBAUD_EN; /* Autoband detect enable register */
    uint32_t HIGHSPEED; /* High speed mode register */
    uint32_t SAMPLE_COUNT; /* Sample counter register */
    uint32_t SAMPLE_POINT; /* Sample point register */
    uint32_t AUTOBAUD_REG; /* Auto Baud Monitor Register */
    uint32_t RATEFIX_AD; /* Clock Rate Fix Register */
    uint32_t AUTOBAUDSAMPLE; /* Auto Baud Sample Register */
    uint32_t GUARD; /* Guard Time Added reRegister */
    uint32_t ESCAPE_DAT; /* Escape character register */
    uint32_t ESCAPE_EN; /* Escape enable register */
    uint32_t SLEEP_EN; /* Sleep enable register */
    uint32_t DMA_EN; /* DMA enable register */
    uint32_t RXTRI_AD; /* Rx Trigger Address */
    uint32_t FRACDIV_L; /* Fractional Divider LSB Address */
    uint32_t FRACDIV_M; /* Fractional Divider MSB Address */
    uint32_t FCR_RD; /* FIFO Control Register */
    uint32_t TX_ACTIVE_EN; /* TX Active Enable Address */
} UART_REGISTER_LIST_T;

typedef struct {
    bool is_rx;
    hal_uart_port_t uart_port;
} uart_dma_callback_data_t;

#ifdef HAL_UART_FEATURE_SOFT_DTE_SUPPORT
/* event generated by DTE pins change */
typedef enum {
    HAL_UART_DTE_EVENT_DSR_ACTIVE = 0,
    HAL_UART_DTE_EVENT_DSR_DEACTIVE = 1,
    HAL_UART_DTE_EVENT_DCD_ACTIVE = 2,
    HAL_UART_DTE_EVENT_DCD_DEACTIVE = 3,
    HAL_UART_DTE_EVENT_RI_HIGH = 4,
    HAL_UART_DTE_EVENT_RI_LOW = 5
} hal_uart_dte_callback_event_t;

/* declare for user defined callback */
typedef void (*hal_uart_dte_callback_t)(hal_uart_dte_callback_event_t event, void *user_data);

typedef struct {
    uint32_t dsr_debounce_time;
    uint32_t dcd_debounce_time;
    uint32_t ri_debounce_time;
    bool dsr_is_active_high;
    bool dcd_is_active_high;
    bool dtr_is_active_high;
    hal_uart_dte_callback_t user_callback;
    void *user_data;
} hal_uart_dte_config_t;

typedef struct {
    hal_uart_port_t uart_port;
    hal_gpio_pin_t ri_pin;
    hal_gpio_pin_t dsr_pin;
    hal_gpio_pin_t dcd_pin;
    hal_gpio_pin_t dtr_pin;
    hal_eint_number_t dsr_eint;
    hal_eint_number_t ri_eint;
    hal_eint_number_t dcd_eint;
    hal_uart_dte_config_t dte_config;
} uart_dte_config_t;

hal_uart_status_t hal_uart_set_dte_config(hal_uart_port_t uart_port,
        hal_uart_dte_config_t *uart_dte_config);
hal_uart_status_t hal_uart_set_dte_dtr_active(hal_uart_port_t uart_port);
hal_uart_status_t hal_uart_set_dte_dtr_deactive(hal_uart_port_t uart_port);
#endif

#define UART_DMA_MAX_SETTING_VALUE (2<<16)
#define UART_ASSERT() assert(0)

void uart_reset_default_value(UART_REGISTER_T *uartx);
void uart_query_empty(UART_REGISTER_T *uartx);
void uart_set_high_baudrate(UART_REGISTER_T *uartx, uint32_t uart_clock, uint32_t baudrate);
void uart_set_low_baudrate(UART_REGISTER_T *uartx, uint32_t uart_clock, uint32_t baudrate);
void uart_set_baudrate(UART_REGISTER_T *uartx, uint32_t actual_baudrate);
void uart_set_format(UART_REGISTER_T *uartx,
                     hal_uart_word_length_t word_length,
                     hal_uart_stop_bit_t stop_bit,
                     hal_uart_parity_t parity);
void uart_put_char_block(UART_REGISTER_T *uartx, uint8_t byte);
uint8_t uart_get_char_block(UART_REGISTER_T *uartx);
void uart_set_hardware_flowcontrol(UART_REGISTER_T *uartx);
void uart_set_software_flowcontrol(UART_REGISTER_T *uartx,
                                   uint8_t xon,
                                   uint8_t xoff,
                                   uint8_t escape_character);
void uart_disable_flowcontrol(UART_REGISTER_T *uartx);
#ifdef HAL_SLEEP_MANAGER_ENABLED
void uart_unmask_send_interrupt(UART_REGISTER_T *uartx);
void uart_set_sleep_mode(UART_REGISTER_T *uartx);
#endif
void uart_set_fifo(UART_REGISTER_T *uartx);
vdma_channel_t uart_port_to_dma_channel(hal_uart_port_t uart_port, int32_t is_rx);
void uart_enable_dma(UART_REGISTER_T *uartx);
void uart_unmask_receive_interrupt(UART_REGISTER_T *uartx);
void uart_purge_fifo(UART_REGISTER_T *uartx, int32_t is_rx);
uart_interrupt_type_t uart_query_interrupt_type(UART_REGISTER_T *uartx);
int32_t uart_verify_error(UART_REGISTER_T *uartx);
void uart_clear_timeout_interrupt(UART_REGISTER_T *uartx);
void uart_send_handler(hal_uart_port_t uart_port, bool is_send_complete_trigger);
void uart_receive_handler(hal_uart_port_t uart_port, bool is_timeout);
void uart_error_handler(hal_uart_port_t uart_port);

#ifdef __cplusplus
}
#endif

#endif

#endif

